
#ifndef MAINWINDOW_H
#define MAINWINDOW_H

#include <QMainWindow>
#include "ui_mainwindow.h"

#include <QGraphicsView>
#include <QGraphicsSceneMouseEvent>
#include <QGraphicsTextItem>
#include <QGraphicsLineItem>
#include <QGraphicsRectItem>
#include <QGraphicsItemGroup>
//#include <QImage>
#include <QProcess>
#include <QtMultimedia/QMediaPlayer>
#include <QAudioOutput>
#include <QTimer>

#include "complexe.h"


struct ENR_FFT
{
    double amplitude; // 8 octets
    uint32_t t; // 4 octets
    uint8_t num; // 1 octet
    uint8_t midi; //1 octet
// total 4+1+1+8 = 14 octets
};


// à propos des notes : PRINCIPE
// les notes ont une fréquence qui n'est pas directement enregistrée ici mais est représenté par le numéro MIDI.
// elles ont un emplacement qui est enregistré et qui permet de connaitre l'instant de son jeu, connaissant la grille.
// pour la grille temporelle, voir 'tracer_grille_MESURES()'
// la variable 'duree' pourrait être calculée en fonction des précédentes, mais le fait de l'avoir ici
// permet de visualiser une toute partie de la partition, voir une note unique, en dehors du contexte
struct NOTE
{
    uint16_t numero;  // 2 octets
    uint8_t midi;     // 1 octet
    uint16_t n_barre; // 2 octets ; numéro de la barre temporelle sur laquelle est posée la note
    uint8_t n_temps;  // 1 octet  ; numéro du temps (temps fort, temps faible) au sein de la mesure
    uint8_t duree;    // 1 octet  ; en nb d'intervalles de barres
// total 7 octets
};



/* FFT */

class MainWindow : public QMainWindow, public Ui::MainWindow
{
    Q_OBJECT
    
public:
    explicit MainWindow(QWidget *parent = 0);
    ~MainWindow();

protected:

    void mousePressEvent(QMouseEvent *event);


private slots:
    void Tic1();
    void Tic2();
    void save_fichier_ini();
    void load_fichier_ini();
    void choix_dossier_de_travail();
    void open_fichier_wav();

    void on_Bt_load_wav_clicked();
    void load_fichier_FRQ();
    void save_fichier_FRQ();
    void load_fichier_NOTES();
    void save_fichier_NOTES();

    void decode_entete();
    void garnis_temp_data(qint32 offset);

    void init_TW_1();
    void init_TAB_FRQ();
    void init_TAB_lst_notes();
    void init_TW_type_M();
    void init_Autres();

    void Etiquette(int x, int y, QString s, QString coul_txt, QString coul_fond, QString coul_cadre , QGraphicsItemGroup *calque_i);

    void afficher_titre_calque(QString titre, int x, int y, QGraphicsItemGroup *calque_i);
    void tracer_graduations_signal();
    void tracer_graduations_FFT();
    void tracer_signal_complet();
    void tracer_gradu_temporelle_signal_entree();
    void tracer_signal_a_convertir();
    void tracer_graduation_TAB_NOTES();
    void tracer_echelle_temps_TAB_NOTES();
    float calcul_x_barre(int n_barre);
    void tracer_grille_MESURES();
    void tracer_1enr_sur_TAB_NOTES(ENR_FFT enr_i);
    void retracer_TAB_FRQ();
    void maj_TAB_NOTES();

    void encadrement_signal(int x, int dx);
    void encadrement_note(int n_midi);
    void encadre_midi(int n_midi);
    int calcul_num_midi(double x);
    void tracer_notes_FFT();

    void complete_case(int row, int column, QString s1, QString c1, QString c2, bool bold1, QTableWidget *QTI);
    void affiche_liste_notes();
    void trace_liste_notes();
    int test_if_note_in_liste(uint16_t n_barre, int midi);
    void affi_txt_en_couleur(int midi_i, QLineEdit *lineEdit_i);

    void ajout_note(int midi, int B, int T);
    void supprime_note(uint16_t n);

    void remplir_tableau_echantillons_sinus();
    void remplir_tableau_echantillons_signal();
    void calcul_compression();
    void bit_reverse_tableau_X();

    int calcul_y_note(int mid); // en pixels sur la grille
    int calcul_hauteur_note(int mid);
    void calcule_durees_notes();
    void calcul_tableau_W();
    void calcul_FFT();
    void calcul_ofsset();
    void traitement_signal();
    void analyse_tout();
    QString nom_note(int mid);

    void numerotation_liste_notes();
    void tri_liste_notes_par_num_barre();
    void suppression_notes_invalides();
    void suppression_mesures();

    void play_note(int midi);// sortie audio
    int joue_1_note_de_la_liste();
    void surbrille_note(uint16_t n);

    void effacer_calque(QGraphicsScene *scene_i, QGraphicsItemGroup *calque_i);

    void on_spinBox_frq_valueChanged(int arg1);
    void on_checkBox1_toggled(bool checked);
    void on_checkBox2_toggled(bool checked);
    void on_spinBox_frq_2_valueChanged(int arg1);
    void on_checkBox3_toggled(bool checked);
    void on_checkBox4_clicked(bool checked);
    void on_spinBox_frq_3_valueChanged(int arg1);

   // void on_toolButton_2_clicked();
    void on_spinBox_1_valueChanged(int arg1);
    void on_spinBox_2_valueChanged(int arg1);
    void on_spinBox_3_valueChanged(int arg1);
    void on_spinBox_4_valueChanged(int arg1);
    void on_spinBox_5_valueChanged(int arg1);

    //void on_spinBox_6_valueChanged(int arg1);
    void on_spinBox_6_valueChanged();

    void on_spinBox_7_valueChanged(int arg1);
    void on_pushButton_8_clicked();
    void on_doubleSpinBox_1_valueChanged();
    void on_Bt_RAZ_clicked();
    void on_Bt_scan_auto_clicked();
    void on_pushButton_2_clicked();
    void on_Btn_52_clicked();
    void on_Btn_94_clicked();
    //void on_spinBox_n_midi_valueChanged(int arg1);
    void on_pushButton_3_clicked();
    void on_Bt_close_entete_clicked();
    void on_Bt_close_frame1_clicked();
    void on_pushButton_4_clicked();
    void on_Bt_save_clicked();
    void on_pushButton_6_clicked();
    void on_Bt_efface_clicked();
    void on_pushButton_5_clicked();
    void on_checkBox_rapide_stateChanged(int arg1);
    void on_doubleSpinBox_seuil_valueChanged(double arg1);
    void on_checkBox_norm_clicked();

    void on_Bt_jouer_tout_clicked();
    void on_Bt_toggle_visu_notes_clicked();
    void on_Bt_save_notes_clicked();
    void on_Bt_load_notes_clicked();
    void on_Bt_joue_passage_clicked();
    void on_Bt_joue_wav_clicked();
    void on_Btn_RAZ_notes_clicked();
    void on_Bt_stop_clicked();
    void on_Bt_stop_2_clicked();

    void on_Bt_mode_R_clicked();
    void on_Bt_mode_W_clicked();
    void on_Btn_SUPP_notes_invalides_clicked();
    void on_Bt_toggle_visu_freq_clicked();
    void on_Bt_efface_2_clicked();
    void on_Bt_toggle_visu_ech_tps_clicked();
    void on_Bt_toggle_visu_mesures_clicked();
    void on_Bt_joue_wav_2_clicked();
    void on_Bt_choix_current_dir_clicked();
    void on_pushButton_9_clicked();

    void on_spinBox_8_textChanged();
    void on_doubleSpinBox_dt_valueChanged();
    void on_doubleSpinBox_t0_valueChanged();
    void on_spinBox_offset_valueChanged();
    void on_spinBox_zoom_2_valueChanged(int arg1);
    void on_spinBox_d_barres_valueChanged();

    void on_Bt_load_FREQ_clicked();

    void on_Bt_decale_notes_L_clicked();
    void on_Bt_decale_notes_R_clicked();

    void on_tableWidget_type_M_cellClicked(int row, int column);

    void on_checkBox_flitrer_clicked();
    //void on_spinBox_filtre_valueChanged();
    //void on_doubleSpinBox_gain_valueChanged();
    void on_Bt_open_DIR_clicked();
    void on_Bt_goto_clicked();

    void on_doubleSpinBox_x0_valueChanged(double arg1);
    void on_doubleSpinBox_T0_valueChanged(double arg1);

    void on_Bt_test1_clicked();

    void on_Bt_div2_clicked();

    void on_pushButton_clicked();

    void on_Bt_next_clicked();

    void on_Bt_prev_clicked();

    void on_Bt_debut_clicked();

    void on_Bt_fin_clicked();

private:

    QPoint Zoom_point2_;

  //  QImage image1;

    QGraphicsScene *scene1; // sur l'onglet FFT - Trace FFT
    QGraphicsItemGroup *calque_reticule1;
    QGraphicsItemGroup *calque_trace_signal1;
    QGraphicsItemGroup *calque_trace_FFT;
    QGraphicsItemGroup *calque_curseur;
    QGraphicsItemGroup *calque_encadrement1;
    QGraphicsItemGroup *calque_lignes_F_zero;

    QGraphicsScene *scene2; // sur l'onglet FFT - Signal d'entrée
    QGraphicsItemGroup *calque_reticule2;
    QGraphicsItemGroup *calque_encadrement2;
    QGraphicsItemGroup *calque_trace_signal2;

    QGraphicsScene *scene3;
    QGraphicsItemGroup *calque_gradu_TAB_FRQ;

    QGraphicsScene *scene4; // sur l'onglet NOTES
    QGraphicsItemGroup *calque_TAB_FRQ;

    QGraphicsItemGroup *calque_grille_mesures;
    QGraphicsItemGroup *calque_echelle_temporelle;
    QGraphicsItemGroup *calque_notes;
    QGraphicsItemGroup *calque_notes_jouee;

    QGraphicsLineItem *ligne1;
    QGraphicsLineItem *segment_trace;
    QGraphicsLineItem *segment_trace2;
    QGraphicsRectItem *rectangle1;
    QGraphicsEllipseItem *ellipse1;
    QGraphicsRectItem *rect1;
    QGraphicsRectItem *rect2;
    QGraphicsRectItem *rect3;

    QGraphicsTextItem *GraphicsTextItem;

    QMediaPlayer *player1;
    QMediaPlayer *player2;
    QMediaPlayer *player3;
    QMediaPlayer *player4;
    QMediaPlayer *player5;

    QAudioOutput  *audioOutput1;
    QAudioOutput  *audioOutput2;
    QAudioOutput  *audioOutput3;
    QAudioOutput  *audioOutput4;
    QAudioOutput  *audioOutput5;

    QTimer *Timer1; // pour analyse auto à faible vitesse
    QTimer *Timer2; // pour rejouer les notes détectées

};



#endif // MAINWINDOW_H
